// rdbutton_panel.cpp
//
// The sound panel widget for RDAirPlay
//
//   (C) Copyright 2002-2003 Fred Gleason <fredg@paravelsystems.com>
//
//      $Id: rdbutton_panel.cpp,v 1.6 2007/02/14 21:48:41 fredg Exp $
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <qcolor.h>

#include <rtransportbutton.h>

#include <rdbutton_panel.h>
#include <rdbutton_dialog.h>


RDButtonPanel::RDButtonPanel(RDStation *station,bool flash,QWidget *parent)
{
  panel_station=station;
  for(int i=0;i<PANEL_BUTTON_ROWS;i++) {
    for(int j=0;j<PANEL_BUTTON_COLUMNS;j++) {
      panel_button[i][j]=new RDPanelButton(panel_station,flash,parent);
      panel_button[i][j]->setGeometry((15+PANEL_BUTTON_SIZE_X)*j,
				      (15+PANEL_BUTTON_SIZE_Y)*i,
				      PANEL_BUTTON_SIZE_X,
				      PANEL_BUTTON_SIZE_Y);
      panel_button[i][j]->hide();
      parent->connect(parent,SIGNAL(buttonFlash(bool)),
		      panel_button[i][j],SLOT(flashButton(bool)));
    }
  }
  clear();
}


QSize RDButtonPanel::sizeHint() const
{
  return QSize(500,535);
}


QSizePolicy RDButtonPanel::sizePolicy() const
{
  return QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
}


RDPanelButton *RDButtonPanel::panelButton(int row,int col) const
{
  return panel_button[row][col];
}


void RDButtonPanel::setActionMode(RDAirPlayConf::ActionMode mode)
{
  switch(mode) {
      case RDAirPlayConf::CopyFrom:
	for(int i=0;i<PANEL_BUTTON_ROWS;i++) {
	  for(int j=0;j<PANEL_BUTTON_COLUMNS;j++) {
	    if(panel_button[i][j]->cart()!=0) {
	      panel_button[i][j]->setColor(BUTTON_FROM_BACKGROUND_COLOR);
	    }
	  }
	}
	break;

      default:
	for(int i=0;i<PANEL_BUTTON_ROWS;i++) {
	  for(int j=0;j<PANEL_BUTTON_COLUMNS;j++) {
	    if(panel_button[i][j]->cart()!=0) {
	      if(panel_button[i][j]->state()) {
		panel_button[i][j]->setColor(RDPANEL_PLAY_BACKGROUND_COLOR);
	      }
	      else {
		panel_button[i][j]->reset();
	      }
	    }
	  }
	}
	break;
  }
}


void RDButtonPanel::hide()
{
  for(int i=0;i<PANEL_BUTTON_ROWS;i++) {
    for(int j=0;j<PANEL_BUTTON_COLUMNS;j++) {
      panel_button[i][j]->hide();
    }
  }
}


void RDButtonPanel::show()
{
  for(int i=0;i<PANEL_BUTTON_ROWS;i++) {
    for(int j=0;j<PANEL_BUTTON_COLUMNS;j++) {
      panel_button[i][j]->show();
    }
  }
}


void RDButtonPanel::clear()
{
  for(int i=0;i<PANEL_BUTTON_ROWS;i++) {
    for(int j=0;j<PANEL_BUTTON_COLUMNS;j++) {
      panel_button[i][j]->clear();
    }
  }
}
